# $Id$
# $URL$

use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Configuration;

use Spiffy -Base;

use IO::File;
use Xyzzy::Static;

field 'generators';
field 'fallback' => 'Xyzzy::Static';

field 'db_type' => 'Pg';
field 'db_name';
field 'db_user';
field 'db_pass';

*set_db_type = *db_type;
*set_db_name = *db_name;
*set_db_user = *db_user;
*set_db_pass = *db_pass;

field 'templatedir';
field 'staticrootdir';

sub db_datasource {
	return $self->{db_datasource} = $_[0] if @_;
	my $db_datasource = $self->{db_datasource};
	return $self->{db_datasource} if exists $self->{db_datasource};

	my $type = $self->db_type;
	my $name = $self->db_name;
	$name = '' unless defined $name;
	return "dbi:$type:dbname=$name"
}

*set_db_datasource = *db_datasource;

our $defaultoptions = {
	PrintError => 0,
	RaiseError => 1,
	ShowErrorStatement => 1,
	pg_enable_utf8 => 1
};

sub db_options {
    return $defaultoptions
}

sub set_templatedir {
	my $dir = $_[0];
	die "Missing argument to TemplateDir\n" unless defined $dir;
	stat $dir;
	warn "TemplateDir $dir doesn't exist\n" unless -e _;
	warn "TemplateDir $dir is not a directory\n" unless -d _;
	$self->templatedir($dir);
}

sub set_staticrootdir {
	my $dir = $_[0];
	die "Missing argument to StaticRootDir\n" unless defined $dir;
	stat $dir;
	warn "StaticRootDir $dir doesn't exist\n" unless -e _;
	warn "StaticRootDir $dir is not a directory\n" unless -d _;
	$self->staticrootdir($dir);
}

sub set_application {
	my $class = $_[0];
	die "Missing argument to Application\n" unless defined $class;
	require $class.'/Configuration.pm';
	bless $self, "${class}::Configuration";
	$self->new;
}

sub handle_configuration_item {
	my ($key, $values, $filename, $lineno) = @_;
	if(defined $key) {
		my $lkey = lc("set_$key");
		if($self->can($lkey)) {
			eval { $self->$lkey(@$values) };
			die "$filename:$lineno: $key: $@" if $@;
		} else {
			die "Unknown key in configuration file $filename:$lineno: $key\n";
		}
	}
}

sub include {
	foreach my $filename (@_) {
		my $fh = new IO::File($filename, '<:utf8')
			or die "Can't open configuration file '$filename': $!\n";;

		my ($ws, $lineno);
		my ($key, @values);

		while(my $line = $fh->getline) {
			next if ord($line) == 35; #
			$line =~ s/\s+$//;
			if($line eq '') {
				$ws = 1;
				next;
			}
			if($line =~ s/^\s//) {
				unless(defined $key) {
					my $lineno = $fh->input_line_number;
					warn "Line continuation before key at $filename:$lineno\n";
					next;
				}
				if($ws) {
					push @values, '';
					undef $ws;
				}
				push @values, $line;
			} else {
				$self->handle_configuration_item($key, \@values, $filename, $lineno);
				@values = ();
				$lineno = $fh->input_line_number;
				($key, my $val) = split(/\s+/, $line, 2);
				push @values, $val;
				undef $ws;
			}	
		}

		die "Can't read from configuration file '$filename': $!\n"
			unless $fh->eof;

		$self->handle_configuration_item($key, \@values, $filename, $lineno);

		$fh->close;
	}
}

*set_include = *include;

sub new {
	return $self if ref $self;
	return bless {}, $self;
}
